/*
 Non-Contact Blackout Detector
 Version: 1.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 8/01/2012
 Last Modified: 4/02/2012

 */

#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL    ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void system_sleep(void) {
	cbi(ADCSRA,ADEN); // Switch Analog to Digital converter OFF
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
	sbi(ADCSRA,ADEN);  // Switch Analog to Digital converter ON
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void setup_watchdog(int ii) {
	byte bb;
	if (ii > 9 ) ii=9;
	bb=ii & 7;
	if (ii > 7) bb|= (1<<5);
	bb|= (1<<WDCE);
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCR = bb;
	WDTCR |= _BV(WDIE);
}

// Turn off the Watchdog
void turnOffwatchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCR |= (1<<WDCE) | (1<<WDE);
	WDTCR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdogSleep(int timeout){ 
	setup_watchdog(timeout);
	system_sleep();
	turnOffwatchdog();
}

// Used from Arduino wiring_analog.c
int analogRead(uint8_t pin) {
	uint8_t low, high;
	
	// set the analog reference (high two bits of ADMUX) and select the
	// channel (low 4 bits).  this also sets ADLAR (left-adjust result)
	// to 0 (the default).
	//	ADMUX = (analog_reference << 6) | (pin & 0x3f); // more MUX
	// sapo per tiny45
	ADMUX = pin & 0x3f;
	
	// without a delay, we seem to read from the wrong channel
	//delay(1);
	
	// start the conversion
	sbi(ADCSRA, ADSC);
	
	// ADSC is cleared when the conversion finishes
	while (bit_is_set(ADCSRA, ADSC));
	
	// we have to read ADCL first; doing so locks both ADCL
	// and ADCH until ADCH is read.  reading ADCL second would
	// cause the results of each conversion to be discarded,
	// as ADCL and ADCH would be locked when it completed.
	low = ADCL;
	high = ADCH;
	
	// combine the two bytes
	return (high << 8) | low;
}

// Used and modified from Arduino wiring.c - to setup the ATtiny
void setup(void) {
	
	sei(); // Turn on interrupts
	
	// Set ADC prescale factor to 8
	// 1 MHz / 8 = 125 KHz, inside the desired 50-200 KHz range.
	sbi(ADCSRA, ADPS1);
	sbi(ADCSRA, ADPS0);
	
	// Enable ADC conversions
	sbi(ADCSRA, ADEN);
}
